@@ -27,8 +27,6 @@
 #include <memory>
 #include <vector>
 
-#include <boost/utility.hpp>
-
 #include "temporal/domain_swap.h"
 #include "temporal/timeline.h"
 #include "temporal/range.h"
@@ -79,11 +77,14 @@ namespace Properties {
 	LIBARDOUR_API extern PBD::PropertyDescriptor<std::string>       tags;
 	LIBARDOUR_API extern PBD::PropertyDescriptor<uint64_t>          reg_group;
 	LIBARDOUR_API extern PBD::PropertyDescriptor<bool>              contents; // type doesn't matter here, used for signal only
+	LIBARDOUR_API extern PBD::PropertyDescriptor<bool>              region_fx; // type doesn't matter here, used for signal only
 };
 
 class Playlist;
 class Filter;
 class ExportSpecification;
+class Plugin;
+class RegionFxPlugin;
 
 enum LIBARDOUR_API RegionEditState {
 	EditChangesNothing = 0,
@@ -107,11 +108,14 @@ class LIBARDOUR_API Region
 {
 public:
 	typedef std::vector<std::shared_ptr<Source> > SourceList;
+	typedef std::list<std::shared_ptr<RegionFxPlugin>> RegionFxList;
 
 	static void make_property_quarks ();
 
 	static PBD::Signal2<void,std::shared_ptr<RegionList>, const PBD::PropertyChange&> RegionsPropertyChanged;
 
+	PBD::Signal0<void> RegionFxChanged;
+
 	typedef std::map <PBD::PropertyChange, RegionList> ChangeMap;
 
 	virtual ~Region();
@@ -139,6 +143,8 @@ public:
 	timepos_t end()        const;
 	timepos_t nt_last()    const { return end().decrement(); }
 
+	virtual timecnt_t tail () const { return timecnt_t (0); }
+
 	timepos_t source_position () const;
 	timecnt_t source_relative_position (Temporal::timepos_t const &) const;
 	timecnt_t region_relative_position (Temporal::timepos_t const &) const;
@@ -178,6 +184,7 @@ public:
 		{
 			Glib::Threads::Mutex::Lock lm (_operation_rgroup_mutex);
 			if (_retained_group_id == 0) {
+				_retained_take_cnt = 0;
 				++_next_group_id;
 				_operation_rgroup_map.clear ();              // this is used for split & paste operations that honor the region's prior grouping
 				_retained_group_id    = _next_group_id << 4; // this is used for newly created regions via recording or importing
@@ -191,6 +198,7 @@ public:
 			if (_clear_on_destruction) {
 				Glib::Threads::Mutex::Lock lm (_operation_rgroup_mutex);
 				_retained_group_id = 0;
+				_next_group_id += _retained_take_cnt;
 				_operation_rgroup_map.clear();
 			}
 		}
@@ -200,9 +208,14 @@ public:
 	static uint64_t next_group_id () { return _next_group_id; }
 	static void set_next_group_id (uint64_t ngid) { _next_group_id = ngid; }
 
-	/* access the retained group-id for actions like Recording, Import */
-	static uint64_t get_retained_group_id () {
-		return _retained_group_id;
+	/* access the retained group-id for actions like Recording, Import.
+	 *
+	 * Note When a single take creates multiple layered regions (e.g. loop recording)
+	 * then the group id need to be bumped for each take
+	 */
+	static uint64_t get_retained_group_id (uint64_t take = 0) {
+		_retained_take_cnt = std::max (_retained_take_cnt, take);
+		return _retained_group_id + (take << 4);
 	}
 
 	/* access the group-id for an operation on a region, honoring the existing region's group status */
@@ -288,8 +301,8 @@ public:
 	 *  OverlapEnd:      the range overlaps the end of this region.
 	 *  OverlapExternal: the range overlaps all of this region.
 	 */
-	Temporal::OverlapType coverage (timepos_t const & start, timepos_t const & end) const {
-		return Temporal::coverage_exclusive_ends (position(), nt_last(), start, end);
+	Temporal::OverlapType coverage (timepos_t const & start, timepos_t const & end, bool with_tail = false) const {
+		return Temporal::coverage_exclusive_ends (position(), with_tail ? nt_last() + tail() : nt_last(), start, end);
 	}
 
 	bool exact_equivalent (std::shared_ptr<const Region>) const;
@@ -496,6 +509,34 @@ public:
 	void move_cue_marker (CueMarker const &, timepos_t const & region_relative_position);
 	void rename_cue_marker (CueMarker&, std::string const &);
 
+	/* Region Fx */
+	bool load_plugin (ARDOUR::PluginType type, std::string const& name);
+	bool add_plugin (std::shared_ptr<RegionFxPlugin>, std::shared_ptr<RegionFxPlugin> pos = std::shared_ptr<RegionFxPlugin> ());
+	virtual bool remove_plugin (std::shared_ptr<RegionFxPlugin>) { return false; }
+	virtual void reorder_plugins (RegionFxList const&);
+
+	bool has_region_fx () const {
+		Glib::Threads::RWLock::ReaderLock lm (_fx_lock);
+		return !_plugins.empty ();
+	}
+
+	std::shared_ptr<RegionFxPlugin> nth_plugin (uint32_t n) const {
+		Glib::Threads::RWLock::ReaderLock lm (_fx_lock);
+		for (auto const& i : _plugins) {
+			if (0 == n--) {
+				return i;
+			}
+		}
+		return std::shared_ptr<RegionFxPlugin> ();
+	}
+
+	void foreach_plugin (boost::function<void(std::weak_ptr<RegionFxPlugin>)> method) const {
+		Glib::Threads::RWLock::ReaderLock lm (_fx_lock);
+		for (auto const& i : _plugins) {
+			method (std::weak_ptr<RegionFxPlugin> (i));
+		}
+	}
+
 protected:
 	virtual XMLNode& state () const;
 
@@ -521,8 +562,11 @@ protected:
 	}
 
 protected:
+	virtual bool _add_plugin (std::shared_ptr<RegionFxPlugin>, std::shared_ptr<RegionFxPlugin>, bool) { return false; }
+	virtual void fx_latency_changed (bool no_emit);
+	virtual void fx_tail_changed (bool no_emit);
 
-	void send_change (const PBD::PropertyChange&);
+	virtual void send_change (const PBD::PropertyChange&);
 	virtual int _set_state (const XMLNode&, int version, PBD::PropertyChange& what_changed, bool send_signal);
 	virtual void set_position_internal (timepos_t const & pos);
 	virtual void set_length_internal (timecnt_t const &);
@@ -539,6 +583,11 @@ protected:
 
 	DataType _type;
 
+	mutable Glib::Threads::RWLock _fx_lock;
+	uint32_t                      _fx_latency;
+	uint32_t                      _fx_tail;
+	RegionFxList                  _plugins;
+
 	PBD::Property<bool>      _sync_marked;
 	PBD::Property<bool>      _left_of_split;
 	PBD::Property<bool>      _right_of_split;
@@ -617,6 +666,7 @@ private:
 		Explicit  = 0x1, // the user has explicitly grouped or ungrouped this region. explicitly grouped regions can cross track-group boundaries
 	};
 	static uint64_t _retained_group_id;
+	static uint64_t _retained_take_cnt;
 	static uint64_t _next_group_id;
 
 	static Glib::Threads::Mutex         _operation_rgroup_mutex;
